/* Copyright (C) 2006 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "operations.h"
#include <QInputDialog>
#include <QTextStream>
#include <QWorkspace>
#include <QPushButton>
#include <QClipboard>
#include <QVBoxLayout>
#include <QLabel>
#include <QTextBrowser>
#include <QFileDialog>

Operations::Operations(QObject *parent, BaseWidget **active_widget, MainWindow *main_window):QObject(parent)
{
	this->active_widget=active_widget;
	this->main_window=main_window;
	
	//Matrix signals
	connect(main_window->actionInverse, SIGNAL(triggered()), this, SLOT(inverse_callback()));
	connect(main_window->actionDeterminant, SIGNAL(triggered()), this, SLOT(determinant_callback()));
	connect(main_window->actionEigenvalues, SIGNAL(triggered()), this, SLOT(eigenvalues_callback()));
	connect(main_window->actionTranspose, SIGNAL(triggered()), this, SLOT(transpose_callback()));
	connect(main_window->actionSubmatrix, SIGNAL(triggered()), this, SLOT(submatrix_callback()));
	
	//Statistics signals
	connect(main_window->actionMean, SIGNAL(triggered()), this, SLOT(mean_callback()));
	connect(main_window->actionMedian, SIGNAL(triggered()), this, SLOT(median_callback()));
	connect(main_window->actionStd, SIGNAL(triggered()), this, SLOT(std_callback()));
	connect(main_window->actionCov, SIGNAL(triggered()), this, SLOT(cov_callback()));
	connect(main_window->actionCorrcoef, SIGNAL(triggered()), this, SLOT(corrcoef_callback()));
	
	//Plot signals
	connect(main_window->actionPlot, SIGNAL(triggered()), this, SLOT(plot_callback()));
	connect(main_window->actionToEPS, SIGNAL(triggered()), this, SLOT(to_eps_callback()));
	connect(main_window->actionToPDF, SIGNAL(triggered()), this, SLOT(to_pdf_callback()));
	connect(main_window->actionToPNG, SIGNAL(triggered()), this, SLOT(to_png_callback()));
}

void Operations::setOctaveConnection(OctaveConnection *octave_connection)
{
	this->octave_connection=octave_connection;
}

void Operations::setMatrix(QString matrix)
{
	this->matrix=matrix;
}

void Operations::inverse_callback()
{
	BaseWidget *top_widget=*active_widget;
	QString window_name=top_widget->windowTitle();
	bool ok;
	
	if(top_widget->widgetType()==TABLE)
	{
		QString aux;
		QTextStream(&window_name) >> aux >> matrix;
	}
	
	ok=input_dialog("Inverse", "Matrix name:", matrix);
	
	if (ok && !matrix.isEmpty())
	{
		QString command;
		QTextStream(&command) << "inv(" << matrix << ")";
		
		if(copy_clipboard_ok)
		{
			 QClipboard *clipboard = QApplication::clipboard();
			 clipboard->setText(command);
		}
		//else if(window_name.startsWith("Octave Terminal") || window_name.startsWith("Help"))
		else
		{
			octave_connection->command_enter(command);
		}
		//else if(window_name.startsWith("Table:"))
		//{
		//	octave_connection->command_enter(matrix+"="+command);
		//	((Table*)top_widget)->windowActivated(top_widget);
		//}
	}
}

void Operations::determinant_callback()
{
	BaseWidget *top_widget=*active_widget;
	QString window_name=top_widget->windowTitle();
	bool ok;
	
	if(top_widget->widgetType()==TABLE)
	{
		QString aux;
		QTextStream(&window_name) >> aux >> matrix;
	}
	
	ok=input_dialog("Determinant", "Matrix name:", matrix);
	
	if (ok && !matrix.isEmpty())
	{
		QString command;
		QTextStream(&command) << "det(" << matrix << ")";
		
		if(copy_clipboard_ok)
		{
			 QClipboard *clipboard = QApplication::clipboard();
			 clipboard->setText(command);
		}
		else octave_connection->command_enter(command);
	}
}

void Operations::eigenvalues_callback()
{
	BaseWidget *top_widget=*active_widget;
	QString window_name=top_widget->windowTitle();
	
	if(top_widget->widgetType()==TABLE)
	{
		QString aux;
		QTextStream(&window_name) >> aux >> matrix;
	}
	
	copy_clipboard_ok=false;
	
	//Se muestra el cuadro de dilogo.
	QDialog dialog(main_window);
	dialog.setWindowTitle("Eigenvalues and eigenvectors");
	QVBoxLayout layout;
	dialog.setLayout(&layout);
	
	QLabel matrix_label("Matrix name:",&dialog);
	layout.addWidget(&matrix_label);
	matrix_label.show();
	
	QLineEdit matrix_lineedit(&dialog);
	matrix_lineedit.setText(matrix);
	layout.addWidget(&matrix_lineedit);
	matrix_lineedit.show();
	
	QLabel eigenvalues_label("Eigenvalues matrix name:",&dialog);
	layout.addWidget(&eigenvalues_label);
	eigenvalues_label.show();
	
	QLineEdit eigenvalues_lineedit(&dialog);
	eigenvalues_lineedit.setText("eigenvalues");
	layout.addWidget(&eigenvalues_lineedit);
	eigenvalues_lineedit.show();
	
	QLabel eigenvectors_label("Eigenvectors matrix name:",&dialog);
	layout.addWidget(&eigenvectors_label);
	eigenvectors_label.show();
	
	QLineEdit eigenvectors_lineedit(&dialog);
	eigenvectors_lineedit.setText("eigenvectors");
	layout.addWidget(&eigenvectors_lineedit);
	eigenvectors_lineedit.show();
	
	QHBoxLayout buttons_layout;
	layout.addLayout(&buttons_layout);
	QPushButton ok_button("Ok", &dialog);
	buttons_layout.addWidget(&ok_button);
	ok_button.show();
	connect(&ok_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	
	QPushButton copy_clipboard_button("Copy to clipboard", &dialog);
	buttons_layout.addWidget(&copy_clipboard_button);
	copy_clipboard_button.show();
	connect(&copy_clipboard_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	connect(&copy_clipboard_button,SIGNAL(clicked()),this,SLOT(copy_clipboard_callback()));
	
	QPushButton cancel_button("Cancel", &dialog);
	buttons_layout.addWidget(&cancel_button);
	cancel_button.show();
	connect(&cancel_button,SIGNAL(clicked()),&dialog,SLOT(reject()));
	
	dialog.exec();
	
	matrix=matrix_lineedit.text();
	QString eigenvalues=eigenvalues_lineedit.text();
	QString eigenvectors=eigenvectors_lineedit.text();
	
	if (dialog.result()==QDialog::Accepted && !matrix.isEmpty() && !eigenvectors.isEmpty() && ! eigenvalues.isEmpty())
	{
		QString command;
		QTextStream(&command) << "[" << eigenvectors << "," << eigenvalues << "]=eig(" << matrix << ")";
		
		if(copy_clipboard_ok)
		{
			 QClipboard *clipboard = QApplication::clipboard();
			 clipboard->setText(command);
		}
		else
		{
			octave_connection->command_enter(command);
		}
	}
}

void Operations::transpose_callback()
{
	BaseWidget *top_widget=*active_widget;
	QString window_name=top_widget->windowTitle();
	bool ok;
	
	if(top_widget->widgetType()==TABLE)
	{
		QString aux;
		QTextStream(&window_name) >> aux >> matrix;
	}
	
	ok=input_dialog("Transpose", "Matrix name:", matrix);
	
	if (ok && !matrix.isEmpty())
	{
		QString command;
		QTextStream(&command) << matrix << "'";
		
		if(copy_clipboard_ok)
		{
			 QClipboard *clipboard = QApplication::clipboard();
			 clipboard->setText(command);
		}
		else // if(window_name.startsWith("Octave Terminal") || window_name.startsWith("Help"))
		{
			octave_connection->command_enter(command);
		}
		//else if(window_name.startsWith("Table:"))
		//{
		//	octave_connection->command_enter(matrix+"="+command);
		//	((Table*)top_widget)->windowActivated(top_widget);
		//}
	}
}

void Operations::submatrix_callback()
{
	BaseWidget *top_widget=*active_widget;
	QString window_name=top_widget->windowTitle();
	
	if(top_widget->widgetType()==TABLE)
	{
		QString aux;
		QTextStream(&window_name) >> aux >> matrix;
	}
	
	copy_clipboard_ok=false;
	
	//Se muestra el cuadro de dilogo.
	QDialog dialog(main_window);
	dialog.setWindowTitle("Submatrix");
	QVBoxLayout layout;
	dialog.setLayout(&layout);
	
	QLabel matrix_label("Matrix name:",&dialog);
	layout.addWidget(&matrix_label);
	matrix_label.show();
	
	QLineEdit matrix_lineedit(&dialog);
	matrix_lineedit.setText(matrix);
	layout.addWidget(&matrix_lineedit);
	matrix_lineedit.show();
	
	QHBoxLayout rows_layout;
	layout.addLayout(&rows_layout);
	
	QLabel from_row_label("From row:",&dialog);
	rows_layout.addWidget(&from_row_label);
	from_row_label.show();
	
	QLineEdit from_row_lineedit(&dialog);
	from_row_lineedit.setText("1");
	rows_layout.addWidget(&from_row_lineedit);
	from_row_lineedit.show();
	
	QLabel to_row_label("To row:",&dialog);
	rows_layout.addWidget(&to_row_label);
	to_row_label.show();
	
	QLineEdit to_row_lineedit(&dialog);
	to_row_lineedit.setText("1");
	rows_layout.addWidget(&to_row_lineedit);
	to_row_lineedit.show();
	
	QHBoxLayout cols_layout;
	layout.addLayout(&cols_layout);
	
	QLabel from_col_label("From column:",&dialog);
	cols_layout.addWidget(&from_col_label);
	from_col_label.show();
	
	QLineEdit from_col_lineedit(&dialog);
	from_col_lineedit.setText("1");
	cols_layout.addWidget(&from_col_lineedit);
	from_col_lineedit.show();
	
	QLabel to_col_label("To column:",&dialog);
	cols_layout.addWidget(&to_col_label);
	to_col_label.show();
	
	QLineEdit to_col_lineedit(&dialog);
	to_col_lineedit.setText("1");
	cols_layout.addWidget(&to_col_lineedit);
	to_col_lineedit.show();
	
	QHBoxLayout buttons_layout;
	layout.addLayout(&buttons_layout);
	QPushButton ok_button("Ok", &dialog);
	buttons_layout.addWidget(&ok_button);
	ok_button.show();
	connect(&ok_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	
	QPushButton copy_clipboard_button("Copy to clipboard", &dialog);
	buttons_layout.addWidget(&copy_clipboard_button);
	copy_clipboard_button.show();
	connect(&copy_clipboard_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	connect(&copy_clipboard_button,SIGNAL(clicked()),this,SLOT(copy_clipboard_callback()));
	
	QPushButton cancel_button("Cancel", &dialog);
	buttons_layout.addWidget(&cancel_button);
	cancel_button.show();
	connect(&cancel_button,SIGNAL(clicked()),&dialog,SLOT(reject()));
	
	dialog.exec();
	
	matrix=matrix_lineedit.text();
	QString from_row=from_row_lineedit.text();
	QString to_row=to_row_lineedit.text();
	QString from_col=from_col_lineedit.text();
	QString to_col=to_col_lineedit.text();
	
	if (dialog.result()==QDialog::Accepted && !matrix.isEmpty() && !from_row.isEmpty() && !to_row.isEmpty() && !from_col.isEmpty() && !to_col.isEmpty())
	{
		QString command;
		QTextStream(&command) << matrix << "(" << from_row << ":" << to_row << "," << from_col << ":" << to_col <<")";
		
		if(copy_clipboard_ok)
		{
			 QClipboard *clipboard = QApplication::clipboard();
			 clipboard->setText(command);
		}
		else
		{
			octave_connection->command_enter(command);
		}
	}
}


void Operations::copy_clipboard_callback()
{
	copy_clipboard_ok=true;
}

bool Operations::input_dialog(QString title, QString label, QString &input)
{
	copy_clipboard_ok=false;
	
	//Se muestra el cuadro de dilogo.
	QDialog dialog(main_window);
	dialog.setWindowTitle(title);
	QVBoxLayout layout;
	dialog.setLayout(&layout);
	
	QLabel matrix_label(label,&dialog);
	layout.addWidget(&matrix_label);
	matrix_label.show();
	
	QLineEdit matrix_lineedit(&dialog);
	matrix_lineedit.setText(matrix);
	layout.addWidget(&matrix_lineedit);
	matrix_lineedit.show();
	
	QHBoxLayout buttons_layout;
	layout.addLayout(&buttons_layout);
	QPushButton ok_button("Ok", &dialog);
	buttons_layout.addWidget(&ok_button);
	ok_button.show();
	connect(&ok_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	
	QPushButton copy_clipboard_button("Copy to clipboard", &dialog);
	buttons_layout.addWidget(&copy_clipboard_button);
	copy_clipboard_button.show();
	connect(&copy_clipboard_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	connect(&copy_clipboard_button,SIGNAL(clicked()),this,SLOT(copy_clipboard_callback()));
	
	QPushButton cancel_button("Cancel", &dialog);
	buttons_layout.addWidget(&cancel_button);
	cancel_button.show();
	connect(&cancel_button,SIGNAL(clicked()),&dialog,SLOT(reject()));
	
	dialog.exec();
	
	input=matrix_lineedit.text();
	
	if (dialog.result()==QDialog::Accepted && !input.isEmpty()) return true;
	return false;
}

void Operations::simple_comand(QString title, QString label, QString _command)
{
	BaseWidget *top_widget=*active_widget;
	bool ok;
	
	if(top_widget->widgetType()==TABLE)
	{
		matrix=((Table*)top_widget)->getMatrix();
	}
	
	ok=input_dialog(title, label, matrix);
	
	if (ok && !matrix.isEmpty())
	{
		QString command;
		QTextStream(&command) << _command << "(" << matrix << ")";
		
		if(copy_clipboard_ok)
		{
			 QClipboard *clipboard = QApplication::clipboard();
			 clipboard->setText(command);
		}
		else
		{
			octave_connection->command_enter(command);
		}
	}
}

Operations::MenuResult Operations::menu_window(QString title, QStringList parameters_labels, QStringList &parameters, QStringList output_labels, QStringList &output, QString help)
{
	copy_clipboard_ok=false;
	
	//Se muestra el cuadro de dilogo.
	QDialog dialog(main_window);
	dialog.setWindowTitle(title);
	QVBoxLayout layout;
	dialog.setLayout(&layout);
	
	QHBoxLayout parameters_layout[parameters_labels.size()];
	QLabel parameters_label[parameters_labels.size()];
	QLineEdit parameters_lineedit[parameters_labels.size()];
	
	for (int i = 0; i < parameters_labels.size(); ++i)
	{
		layout.addLayout(&parameters_layout[i]);
		
		parameters_label[i].setText(parameters_labels.at(i));
		parameters_label[i].setParent(&dialog);
		parameters_layout[i].addWidget(&parameters_label[i]);
		parameters_label[i].show();
		
		parameters_lineedit[i].setParent(&dialog);
		parameters_lineedit[i].setText(parameters.at(i));
		parameters_layout[i].addWidget(&parameters_lineedit[i]);
		parameters_lineedit[i].show();
	}
	
	QHBoxLayout output_layout[parameters_labels.size()];
	QLabel output_label[parameters_labels.size()];
	QLineEdit output_lineedit[parameters_labels.size()];
	
	for (int i = 0; i < output_labels.size(); ++i)
	{
		layout.addLayout(&output_layout[i]);
		
		output_label[i].setText(output_labels.at(i));
		output_label[i].setParent(&dialog);
		output_layout[i].addWidget(&output_label[i]);
		output_label[i].show();
		
		output_lineedit[i].setParent(&dialog);
		output_lineedit[i].setText(output.at(i));
		output_layout[i].addWidget(&output_lineedit[i]);
		output_lineedit[i].show();
	}
	
	QTextBrowser help_text(&dialog);
	help_text.hide();
	
	if(!help.isEmpty())
	{
		help_text.setHtml(help);
		layout.addWidget(&help_text);
		help_text.show();
	}
	
	QHBoxLayout buttons_layout;
	layout.addLayout(&buttons_layout);
	QPushButton ok_button("Ok", &dialog);
	buttons_layout.addWidget(&ok_button);
	ok_button.show();
	connect(&ok_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	
	QPushButton copy_clipboard_button("Copy to clipboard", &dialog);
	buttons_layout.addWidget(&copy_clipboard_button);
	copy_clipboard_button.show();
	connect(&copy_clipboard_button,SIGNAL(clicked()),&dialog,SLOT(accept()));
	connect(&copy_clipboard_button,SIGNAL(clicked()),this,SLOT(copy_clipboard_callback()));
	
	QPushButton cancel_button("Cancel", &dialog);
	buttons_layout.addWidget(&cancel_button);
	cancel_button.show();
	connect(&cancel_button,SIGNAL(clicked()),&dialog,SLOT(reject()));
	
	dialog.exec();
	
	if (dialog.result()==QDialog::Accepted)
	{
		for (int i = 0; i < output_labels.size(); ++i)
		{
			if(output_lineedit[i].text().isEmpty())
			{
				return CANCEL;
			}
		}
		
		for (int i = 0; i < parameters_labels.size(); ++i)
		{
			if(parameters_lineedit[i].text().isEmpty())
			{
				return CANCEL;
			}
		}
		 
		if(output_labels.size()>0)
		{
			output.clear();
			for (int i = 0; i < output_labels.size(); ++i)
			{
				output << output_lineedit[i].text();
			}
		}
		
		if(parameters_labels.size()>0)
		{
			parameters.clear();
			for (int i = 0; i < parameters_labels.size(); ++i)
			{
				parameters << parameters_lineedit[i].text();
			}
		}
		
		if(copy_clipboard_ok)
		{
			return COPY_CLIPBOARD;
		}
		else
		{
			return ACCEPTED;
		}
	}
	return CANCEL;
}

void Operations::compex_comand(QString title, QString _command, QStringList parameters_labels, QStringList & parameters, QStringList output_labels, QStringList & output, QString help, bool is_table)
{
	MenuResult r=menu_window(title, parameters_labels, parameters, output_labels, output, help);
	
	QString command;
	
	if (r!=CANCEL)
	{
		QTextStream command_stream(&command);
		 
		if(output_labels.size()>0)
		{
			command_stream << "[";
			for (int i = 0; i < output_labels.size();)
			{
				command_stream << output.at(i);
				++i;
				if(i < output_labels.size()) command_stream << ",";
			}
			command_stream << "]=";
		}
		
		if(parameters_labels.size()>0)
		{
			command_stream << _command << "(";
			for (int i = 0; i < parameters_labels.size();)
			{
				command_stream << parameters.at(i);
				++i;
				if(i < parameters_labels.size()) command_stream << ",";
			}
			command_stream << ")";
		}
	}
	else return;
	
	
	if(r==COPY_CLIPBOARD)
	{
			QClipboard *clipboard = QApplication::clipboard();
			clipboard->setText(command);
	}
	//r==ACCEPTED
	else if(!is_table)
	{
		octave_connection->command_enter(command);
	}
	else
	{
		BaseWidget *top_widget=*active_widget;
		matrix=parameters.at(0);
		if(top_widget->widgetType()==TABLE)
		{
			if( matrix==((Table*)top_widget)->getMatrix() )
			{
				command=matrix+"="+command;
			}
		}
		octave_connection->command_enter(command);
	}

}

void Operations::mean_callback()
{
	simple_comand("Mean", "Vector name:", "mean");
}

void Operations::median_callback()
{
	simple_comand("Median", "Vector name:", "median");
}

void Operations::std_callback()
{
	simple_comand("Standard deviation", "Vector name:", "std");
}

void Operations::cov_callback()
{
	QStringList parameters_labels;
	parameters_labels << "x" << "y";
	QStringList parameters;
	parameters << "" << "";
	QStringList output_labels;
	QStringList output;
	QString help("Calculates covariance: cov(x,y)<br>To calculate variance of x use cov(x,x)");
	
	compex_comand("Covariance", "cov", parameters_labels, parameters, output_labels, output, help);
}

void Operations::corrcoef_callback()
{
	QStringList parameters_labels;
	parameters_labels << "x" << "y";
	QStringList parameters;
	parameters << "" << "";
	QStringList output_labels;
	QStringList output;
	
	compex_comand("Correlation Coefficient", "corrcoef", parameters_labels, parameters, output_labels, output);
}

void Operations::plot_callback()
{
	Plot plot(main_window);
	int ok=plot.exec();
	if(ok==QDialog::Accepted)
	{
		QString command=plot.command();
		octave_connection->command_enter(command);
	}
}

void Operations::to_eps_callback()
{
	QString file = QFileDialog::getSaveFileName(main_window,"Choose a file", ".","Postscript (*.eps *.ps)");
	
	if(file.isEmpty()) return;
	
	QString command;
	QTextStream command_stream(&command);
	
	command_stream << "__gnuplot_set__ term post\n__gnuplot_set__ output \"" << file << "\"\nreplot\n__gnuplot_set__ term x11";
	
	octave_connection->command_enter(command);
}

void Operations::to_pdf_callback()
{
	QString file = QFileDialog::getSaveFileName(main_window,"Choose a file", ".","Postscript (*.pdf)");
	
	if(file.isEmpty()) return;
	
	file.replace(" ","\\ ");
	
	QString command;
	QTextStream command_stream(&command);
	
	command_stream << "__gnuplot_set__ term post\n__gnuplot_set__ output \"|ps2pdf - " << file << " \"\nreplot\n__gnuplot_set__ term x11";
	
	octave_connection->command_enter(command);
}

void Operations::to_png_callback()
{
	QString file = QFileDialog::getSaveFileName(main_window,"Choose a file", ".","Postscript (*.png)");
	
	if(file.isEmpty()) return;
	
	QString command;
	QTextStream command_stream(&command);
	
	command_stream << "__gnuplot_set__ term png transparent\n__gnuplot_set__ output \"" << file << "\"\nreplot\n__gnuplot_set__ term x11";
	
	octave_connection->command_enter(command);
}

Plot::Plot(QWidget *parent, Plot::Type type):QDialog(parent)
{
	setWindowIcon( QIcon( ":/images/help.png" ) );
	switch(type)
	{
		case PLOT:
			setWindowTitle("Plot");
			break;
		case SEMILOG:
			setWindowTitle("Semilog");
			break;
	}
	vlayout=new QVBoxLayout();
	//vlayout->setSpacing(0);
	//vlayout->setMargin(0);
	setLayout(vlayout);
	
	//Entradas de texto para x e y
	
	QHBoxLayout *hlayout= new QHBoxLayout();
	vlayout->addLayout(hlayout);
	
	QLabel *x_label = new QLabel("x",this);
	hlayout->addWidget(x_label);
	x_label->show();
	
	x_lineedit= new QLineEdit(this);
	hlayout->addWidget(x_lineedit);
	x_lineedit->show();
	
	hlayout= new QHBoxLayout();
	vlayout->addLayout(hlayout);
	
	QLabel *y_label = new QLabel("y",this);
	hlayout->addWidget(y_label);
	y_label->show();
	
	y_lineedit= new QLineEdit(this);
	hlayout->addWidget(y_lineedit);
	y_lineedit->show();
	
	//Opciones de formato
	
	hlayout= new QHBoxLayout();
	vlayout->addLayout(hlayout);
	
	QLabel *style_label= new QLabel(tr("Line style"),this);
	hlayout->addWidget(style_label);
	style_label->show();
	
	style_combo= new QComboBox(this);
	style_combo->addItem(tr("None"), QVariant(""));
	style_combo->addItem(tr("Lines"), QVariant("-"));
	style_combo->addItem(tr("Dots"), QVariant(""));
	style_combo->addItem(tr("Points"), QVariant("@"));
	style_combo->addItem(tr("Lines and points"), QVariant("-@"));
	style_combo->addItem(tr("Impulses"), QVariant("^"));
	style_combo->addItem(tr("Steps"), QVariant("L"));
	style_combo->addItem(tr("Boxes"), QVariant("#"));
	style_combo->addItem(tr("Error bars"), QVariant("~"));
	style_combo->addItem(tr("Box error bars"), QVariant("#~"));
	hlayout->addWidget(style_combo);
	style_combo->show();
	
	QLabel *color_label= new QLabel(tr("Color"),this);
	hlayout->addWidget(color_label);
	color_label->show();
	
	color_combo= new QComboBox(this);
	color_combo->addItem( tr("None"), QVariant("") );
	color_combo->addItem( tr("Red"), QVariant("r") );
	color_combo->addItem( tr("Green"), QVariant("g") );
	color_combo->addItem( tr("Blue"), QVariant("b") );
	color_combo->addItem( tr("Magenta"), QVariant("m") );
	color_combo->addItem( tr("Cyan"), QVariant("c") );
	color_combo->addItem( tr("White"), QVariant("w") );
	hlayout->addWidget(color_combo);
	color_combo->show();
	
	QLabel *pointStyle_label= new QLabel(tr("Point style"),this);
	hlayout->addWidget(pointStyle_label);
	pointStyle_label->show();
	
	pointStyle_combo= new QComboBox(this);
	pointStyle_combo->addItem( tr("None"), QVariant("") );
	pointStyle_combo->addItem( tr("+"), QVariant("+") );
	pointStyle_combo->addItem( tr("*"), QVariant("*") );
	pointStyle_combo->addItem( tr("o"), QVariant("o") );
	pointStyle_combo->addItem( tr("x"), QVariant("x") );
	hlayout->addWidget(pointStyle_combo);
	pointStyle_combo->show();
	
	//Lista de dibujos aceptados
	
	plots_tree = new QTreeWidget(this);
	plots_tree->setColumnCount(5);
	QStringList headers;
	headers << "x" << "y" << "Line style" << "Color" << "Point style";
	plots_tree->setHeaderLabels(headers);
	vlayout->addWidget(plots_tree);
	plots_tree->show();
	
	//Botones
	
	hlayout= new QHBoxLayout();
	vlayout->addLayout(hlayout);
	
	QPushButton *add_button = new QPushButton(tr("Add"),this);
	hlayout->addWidget(add_button);
	add_button->show();
	
	connect(add_button, SIGNAL(clicked()), this, SLOT(add_callback()) );
	
	QPushButton *remove_button = new QPushButton(tr("Remove"),this);
	hlayout->addWidget(remove_button);
	remove_button->show();
	
	connect(remove_button, SIGNAL(clicked()), this, SLOT(remove_callback()) );
	
	QPushButton *ok_button = new QPushButton(tr("Ok"),this);
	hlayout->addWidget(ok_button);
	ok_button->show();
	
	connect(ok_button,SIGNAL(clicked()),this,SLOT(accept()));
	
	QPushButton *copy_clipboard_button = new QPushButton(tr("Copy to clipboard"),this);
	hlayout->addWidget(copy_clipboard_button);
	copy_clipboard_button->show();
	
	connect(copy_clipboard_button,SIGNAL(clicked()),this,SLOT(copy_clipboard_callback()));

	QPushButton *cancel_button = new QPushButton(tr("Cancel"),this);
	hlayout->addWidget(cancel_button);
	cancel_button->show();
	
	connect(cancel_button,SIGNAL(clicked()),this,SLOT(reject()));
}

void Plot::add_callback()
{
	int style_index=style_combo->currentIndex();
	QString style_text=style_combo->itemText(style_index);
	QVariant style_data=style_combo->itemData(style_index);
	
	int color_index=color_combo->currentIndex();
	QString color_text=color_combo->itemText(color_index);
	QVariant color_data=color_combo->itemData(color_index);
	
	int pointStyle_index=pointStyle_combo->currentIndex();
	QString pointStyle_text=pointStyle_combo->itemText(pointStyle_index);
	QVariant pointStyle_data=pointStyle_combo->itemData(pointStyle_index);
	
	QStringList plot_data;
	plot_data << x_lineedit->text() << y_lineedit->text() << style_text << color_text << pointStyle_text;
	QTreeWidgetItem *item= new QTreeWidgetItem(plots_tree,plot_data);
	
	//Line Style
	item->setData(2,1,style_data);
	//Color
	item->setData(3,1,color_data);
	//Point Style
	item->setData(4,1,pointStyle_data);
}

void Plot::remove_callback()
{
	QTreeWidgetItem *item= plots_tree->currentItem ();
	item=plots_tree-> takeTopLevelItem( plots_tree->indexOfTopLevelItem(item) );
	delete item;
}

QString Plot::command()
{
	QString _command;
	QTextStream command_stream(&_command);
	command_stream << "plot(";
	plots_tree->selectAll();
	QList<QTreeWidgetItem *> list=plots_tree->selectedItems ();
	for (int i = 0; i < list.size(); ++i)
	{
		QTreeWidgetItem *item=list.at(i);
		if(!item->text(0).isEmpty())
			command_stream << item->text(0);
		if(!item->text(1).isEmpty())
		{
			command_stream << "," << item->text(1);
		}
		QString style;
		for(int j=2;j<5;j++)
		{
			if( !item->data(j,1).toString().isEmpty() )
				style+=item->data(j,1).toString();
		}
		if( !style.isEmpty() )
			command_stream << ",\"" << style << "\"";
		if( (i+1)<list.size() )
			command_stream << ",";
		
	}
	command_stream << ")";
	
	return _command;
}

void Plot::copy_clipboard_callback()
{
	QClipboard *clipboard = QApplication::clipboard();
	clipboard->setText( command() );
}

