/* Copyright (C) 2006 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "table.h"
#include <QTextStream>

Table::Table(QWidget * parent):BaseWidget(parent)
{
	widget_type=TABLE;

	table_form=new TableForm();
	table_form->setupUi(this);
	setWindowIcon( QIcon( QString(ICON_PATH)+"/table.png" ) );
	table_form->reloadButton->setIcon( QIcon( QString(ICON_PATH)+"/images/reload.png" ) );
	table_form->reloadButton->setToolTip(tr("<b>Reload matrix.</b><br> Some operations change matrix, use reload to view changes."));
	connect(table_form->table_widget,SIGNAL(cellChanged( int, int)),this,SLOT(cellChanged( int, int)));
	connect(table_form->rows_lineedit,SIGNAL( returnPressed ()),this,SLOT(rows_changed()));
	connect(table_form->rows_lineedit,SIGNAL( clicked() ),this,SLOT( windowActivated() ));
	
	connect(table_form->cols_lineedit,SIGNAL( returnPressed ()),this,SLOT(cols_changed()));
	connect(table_form->cols_lineedit,SIGNAL( clicked() ),this,SLOT( windowActivated() ));
	
	connect(table_form->changeOrderButton,SIGNAL( clicked() ),this,SLOT( rows_changed() ));
	connect(table_form->changeOrderButton,SIGNAL( clicked() ),this,SLOT( cols_changed() ));
	connect(table_form->changeOrderButton,SIGNAL( clicked() ),this,SLOT( windowActivated() ));
	
	connect(table_form->reloadButton,SIGNAL( clicked() ),this,SLOT( windowActivated() ));
}

void Table::setOctaveConnection(OctaveConnection *octave_connection)
{
	this->octave_connection=octave_connection;
	connect(octave_connection,SIGNAL(line_ready(QString)),this,SLOT(line_ready(QString)));
}

void Table::setMatrix(QString matrix)
{
	this->matrix=matrix;
}

void Table::windowActivated()
{
	windowActivated(this);
}

void Table::windowActivated(QWidget *w)
{
	if(w!=this) return;
	QString command;
	command+="function qtoctave_show_matrix(m);";
	command+="fprintf(stderr,\"~~matrix:"+matrix+" #%d %d\\n\",rows(m),columns(m));";
	command+="for i=1:rows(m);for j=1:columns(m);";
	command+="if(imag(m(i,j))==0);";
	command+="fprintf(stderr,\"~~matrix:"+matrix+" %d %d %g\\n\",i,j,m(i,j));";
	command+="elseif(imag(m(i,j))>0);";
	command+="fprintf(stderr,\"~~matrix:"+matrix+" %d %d %g+%gi\\n\",i,j,m(i,j),imag(m(i,j)));";
	command+="else;";
	command+="fprintf(stderr,\"~~matrix:"+matrix+" %d %d %g%gi\\n\",i,j,m(i,j),imag(m(i,j)));";
	command+="endif;";
	command+="endfor;endfor;endfunction;qtoctave_show_matrix("+matrix+")";
	octave_connection->command_enter(command);
}

void Table::line_ready(QString line)
{
	if( ! line.startsWith("~~matrix:"+matrix) ) return;
	line=line.right(line.length() - (10+matrix.length()) );
	//printf("line:>%s<\n",line.ascii());
	if(line.indexOf('#')>-1)
	{	//La lnea contiene el nmero de filas y columnas
		int _rows, _columns;
		sscanf(line.toAscii().data(),"#%d %d",&_rows,&_columns);
		table_form->table_widget->setColumnCount(_columns);
		table_form->table_widget->setRowCount(_rows);
		table_form->rows_lineedit->setText(line.setNum(_rows));
		table_form->cols_lineedit->setText(line.setNum(_columns));
		return;
	}
	else
	{
		//La lnea contiene los datos de la celda
		int row, col;
		char data[256];
		sscanf(line.toAscii().data(),"%d %d %[0-9+iej .-]s",&row,&col,data);
		//printf("(%d,%d) %f",row,col,data);
		
		disconnect(table_form->table_widget,SIGNAL(cellChanged( int, int)),this,SLOT(cellChanged( int, int)));
		QTableWidgetItem *cell= new QTableWidgetItem(data);
		table_form->table_widget->setItem(row-1,col-1,cell);
		connect(table_form->table_widget,SIGNAL(cellChanged( int, int)),this,SLOT(cellChanged( int, int)));
	}
}

void Table::cellChanged ( int row, int col )
{
	QString command(matrix+"("), aux;
	command+=aux.setNum(row+1)+",";
	QTableWidgetItem *item=table_form->table_widget->item(row,col);
	command+=aux.setNum(col+1)+")="+item->text()+";";
	octave_connection->command_enter(command);
}

void Table::change_rows()
{
	int rows=table_form->rows_lineedit->text().toInt();
	int old_rows=table_form->table_widget->rowCount();
	int old_cols=table_form->table_widget->columnCount();
	
	if(rows<old_rows)
	{
		QString command;
		QTextStream(&command) 	<< matrix << "=" << matrix << "(1:" << rows 
					<< ",1:" << old_cols << ");";
		octave_connection->command_enter(command);
	}
	else if(rows>old_rows)
	{
		QString command;
		QTextStream(&command) 	<< matrix << "(" << rows 
					<< "," << old_cols << ")=0;";
		octave_connection->command_enter(command);
	}
}

void Table::rows_changed()
{
	change_rows();
	windowActivated(this);
}

void Table::cols_changed()
{
	change_cols();
	windowActivated(this);
}

void Table::change_cols()
{
	int cols=table_form->cols_lineedit->text().toInt();
	int old_rows=table_form->table_widget->rowCount();
	int old_cols=table_form->table_widget->columnCount();
	
	if(cols<old_cols)
	{
		QString command;
		QTextStream(&command) 	<< matrix << "=" << matrix << "(1:" << old_rows 
					<< ",1:" << cols << ");";
		octave_connection->command_enter(command);
	}
	else if(cols>old_cols)
	{
		QString command;
		QTextStream(&command) 	<< matrix << "(" << old_rows 
					<< "," << cols << ")=0;";
		octave_connection->command_enter(command);
	}
}

void Table::order_changed()
{
	change_rows();
	change_cols();
	windowActivated(this);
}

QString Table::getMatrix()
{
	return matrix;
}


