/* Copyright (C) 2006 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "terminal.h"
#include "config.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTextCursor>
#include <QLineEdit>
#include <QIcon>
#include <QTextStream>

Terminal::Terminal(QWidget * parent):BaseWidget(parent)
{
	//Se crea la ventana del terminal
	setWindowTitle("Octave Terminal");
	setWindowIcon( QIcon(QString(ICON_PATH)+"/images/konsole.png"));
	
	text=new QTextEdit(this);
	text->setLineWrapMode (QTextEdit::NoWrap);
	text->setReadOnly (true);
	text->setFontFamily ("Courier 10 Pitch");
	text->show();
	
	combo_box=new QComboBox(this);
	combo_box->setEditable(true);
	combo_box->setAutoCompletion(false);
	combo_box->setDuplicatesEnabled(true);
	combo_box->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
	combo_box->show();
	//Se captura el "intro" para pasar comandos al octave
	QLineEdit *line_edit=combo_box->lineEdit();
	QObject::connect(line_edit, SIGNAL(returnPressed()), this, SLOT(return_pressed()));
	
	completion_matches_button=new QPushButton(this);
	completion_matches_button->setIcon(QIcon(QString(ICON_PATH)+"/images/search.png"));
	completion_matches_button->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
	completion_matches_button->setToolTip(
		tr("<b>Completion matches:</b><br> Generate possible completions given in comand line.")
		);
	completion_matches_button->show();
	connect(completion_matches_button, SIGNAL(clicked()), this, SLOT(completion_matches_callback()));
	
	QVBoxLayout *layout = new QVBoxLayout;
	QHBoxLayout *comand_layout = new QHBoxLayout;
	layout->addWidget(text);
	layout->addLayout(comand_layout);
	comand_layout->addWidget(completion_matches_button);
	comand_layout->addWidget(combo_box);
	
	setLayout(layout);
}

void Terminal::return_pressed()
{
	QLineEdit *line_edit=combo_box->lineEdit();
	QString command(line_edit->text());
	command_enter(command);
};

QTextEdit *Terminal::getTextEdit()
{
	return text;
}

void Terminal::setOctaveConnection(OctaveConnection *octave_connection)
{
	this->octave_connection=octave_connection;
	//octave_connection->setTextEdit(text);
	QObject::connect(octave_connection, SIGNAL(error_ready(QString)), this, SLOT(write_error(QString)));
	QObject::connect(octave_connection, SIGNAL(output_ready(QString)), this, SLOT(write_output(QString)));
	QObject::connect(octave_connection, SIGNAL(command_ready(QString)), this, SLOT(write_command(QString)));
}

OctaveConnection *Terminal::getOctaveConnection()
{
	return octave_connection;
}

void Terminal::command_enter (const QString &command)
{
	octave_connection->command_enter(command);
	combo_box->clearEditText();
}

void Terminal::write_output(QString output)
{
	QTextCursor cursor=text->textCursor();
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
	text->setTextColor( Qt::black );
	text->setFontWeight ( QFont::Normal );
	text->insertPlainText(output);
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
}

void Terminal::write_error(QString error)
{
	QTextCursor cursor=text->textCursor();
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
	text->setTextColor( Qt::red );
	text->setFontWeight ( QFont::Normal );
	text->insertPlainText(error);
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
}

void Terminal::write_command(QString command)
{
	QTextCursor cursor=text->textCursor();
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
	text->setTextColor( Qt::black );
	text->setFontWeight ( QFont::Bold );
	text->insertPlainText(command);
	text->setFontWeight ( QFont::Normal );
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
}

void Terminal::completion_matches_callback()
{
	QString command;
	QTextStream(&command) << "completion_matches(\"" << combo_box->lineEdit()->text() << "\")";
	
	octave_connection->command_enter(command,false);
}

